use std::process::ExitCode;

use clap::{Parser, Subcommand};
use clap_cargo::style::CLAP_STYLING;

mod console;
mod install;
mod settings;

use console::{Color, error};
use install::Install;
use settings::get_settings;

#[derive(Debug, Parser)]
#[command(version, about, long_about = None)]
#[command(styles = CLAP_STYLING)]
struct Cli {
    /// when to use terminal colors
    #[arg(short, long, value_enum, default_value_t = Color::Auto)]
    color: Color,

    #[command(subcommand)]
    command: Command,
}

#[derive(Debug, Subcommand)]
enum Command {
    Install(Install),
}

impl Command {
    fn run(self) -> anyhow::Result<ExitCode> {
        let settings = get_settings()?;

        match self {
            Self::Install(args) => args.run(settings),
        }
    }
}

fn main() -> ExitCode {
    let Cli { color, command } = Cli::parse();

    console::init(color);

    command.run().unwrap_or_else(|e| {
        error!("{e}");
        ExitCode::FAILURE
    })
}
